#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include "lcd.h"

static unsigned char x_ship = 40; /* ship coordinates */
static unsigned char x_proj, y_proj = 41; /* projectile coordinates */

static unsigned char x_enemy = 41, y_enemy = 5; /* enemy coordinates */

/* enemy movement mechanism */
static char x_e_step;
static unsigned char x_e_counter, y_e_counter;

/* enemy representation */
static unsigned char enemymap[3][9]; /* 0 = mort, 1 = enemyA, 2 = enemyB */

/* nr enemies alive */
static unsigned char enemies_alive;


/* 5x5 representation of battleship */
char battleship[5][5] =  {{0,0,0,0,0},
						  {0,0,1,0,0},
						  {0,1,1,1,0},
						  {1,1,1,1,1},
						  {0,1,1,1,0}};

/* 5x5 representation of projectile */
char projectile[5][5] =  {{0,0,1,0,0},
						  {0,0,1,0,0},
						  {0,1,1,1,0},
						  {1,1,1,1,1},
						  {0,0,1,0,0}};
						  
/* 5x5 representation of the enemies */						  
char enemyA[5][5] = 	 {{0,0,0,0,0},
						  {1,1,1,1,1},
						  {0,1,1,1,0},
						  {1,1,1,1,1},
						  {0,0,1,0,0}};
						  
						  
char enemyB[5][5] = 	 {{1,1,0,1,1},
						  {0,1,1,1,0},
						  {0,0,1,0,0},
						  {1,1,1,1,1},
						  {0,1,0,1,0}};


/* draws the battleship at the given coordinates */
void drawBattleship(unsigned char x, unsigned char y)
{
	char i, j;
	
	
	lcd_goto_xy_exact(0,0);
	for(i = 0; i < 5; i++)
		for(j = 0; j < 5; j++)
			if(battleship[i][j] == 1)
				lcd_setpixel(x + j,y + i);
	
}

/* draws the projectile at the given coordinates */
void drawProjectile(unsigned char x, unsigned char y)
{
	char i, j;
	
	if (y_proj <= 40)
	{
		lcd_goto_xy_exact(0,0);
		for(i = 0; i < 5; i++)
			for(j = 0; j < 5; j++)
				if(projectile[i][j] == 1)
					lcd_setpixel(x + j,y + i);
	}
}


/* checks input from the buttons and moves the ship accordingly */
void move_ship()
{
	if (x_ship > 0)
		if (PINC & (1 << PC1))
			x_ship--;
			
	if (x_ship < 80)
		if (PINC & (1 << PC3))
			x_ship++;	
	
}

/* checks input from button and if that input is 1 it launches the projectile */
void launch_projectile()
{
	if (y_proj > 40)
		if (!(PINC & (1 << PC2)))
		{
			x_proj = x_ship;
			y_proj = 40;
		}
}

/* if the projectile is launched(has a y value between 40 and 0) it animates
	the movement */
void move_projectile()
{
	if (y_proj <= 40)
	y_proj--;
}

/* initializes the buttons */
void initButtons()
{
	DDRC &= ~(1 << PC1);  
	PORTC |= (1 << PC1);  
	
	DDRC &= ~(1 << PC2);  
	PORTC |= (1 << PC2);  
	
	DDRC &= ~(1 << PC3);  
	PORTC |= (1 << PC3);  
}


/*
 Enemy functions
*/

/* initializes the enemies for the start of the game */
void initEnemies()
{
	int i, j;
	
	x_enemy = 41;
	y_enemy = 5;
	x_e_counter = 0;
	x_e_step = 1;
	y_e_counter = 0;
	
	for(i = 0; i < 3; i++)
		for(j = 0; j < 9; j++)
			enemymap[i][j] = 1;
		
	enemymap[0][1] = 2;
	enemymap[0][3] = 2;
	enemymap[0][5] = 2;
	enemymap[0][7] = 2;
	enemymap[1][5] = 2;
	enemymap[1][3] = 2;
	
	enemies_alive = 27;
}

/* draws the enemies */
void drawEnemies()
{
	int i, j, x, y, k, l;
	
	for(i = 0; i < 3; i++)
		for(j = 0; j < 9; j++)
		{
			x = x_enemy - (4 - j) * 8;
			y = y_enemy + 8 * i;
			
			lcd_goto_xy_exact(0,0);
			
			/* draws accordingly to enemy type */
			if(enemymap[i][j] == 1)
			for(k = 0; k < 5; k++)
				for(l = 0; l < 5; l++)
					if(enemyA[k][l] == 1)
						lcd_setpixel(x + l,y + k);
			
			if(enemymap[i][j] == 2)
			for(k = 0; k < 5; k++)
				for(l = 0; l < 5; l++)
					if(enemyB[k][l] == 1)
						lcd_setpixel(x + l,y + k);
		}
}

/* Checks if an enemy and a projectile have collided. If so it
deletes both. */
void collision()
{
	int i, j;
	
	if(y_proj <= y_enemy + 2 * 8)
	{
			for(j = 0; j < 3; j++)
			{
				for(i = 0; i < 9; i++)
				if(y_proj < y_enemy + 8 * j && y_proj > y_enemy + 8 * (j - 1))
				if(enemymap[j][i] != 0)
				if(x_proj > x_enemy - (4 - i) * 8 - 2 && x_proj < x_enemy - (4 - i) * 8 + 4)
				{
					enemymap[j][i] = 0;
					y_proj = 41;
					enemies_alive--;
				}
			}
	}
}


/* simulates the movement of the enemies. they move 2 times to the right
then 2 times to the left*/
void move_enemies()
{
	if(x_e_step > 0 && x_e_counter == 20)
		x_enemy += 2 * x_e_step;
	if(x_e_step > 0 && x_e_counter > 40)
	{
		x_enemy += 2 * x_e_step;
		x_e_step *= -1;
	}
	
	if(x_e_step < 0 && x_e_counter == 20)
		x_enemy += 2 * x_e_step;
	if(x_e_step < 0 && x_e_counter < 0)
	{
		x_enemy += 2 * x_e_step;
		x_e_step *= -1;
	}
	
	x_e_counter += x_e_step;
}

/* displays a start screen. waits until the user presses a key */
void start_screen()
{
	char x = 0;
	lcd_clear();
	lcd_goto_xy_exact(0,0);
	lcd_goto_xy(4,1);
	lcd_chr(ARROW_LEFT);
	printf(" Start Joc ");
	lcd_chr(ARROW_RIGHT);
	
	lcd_goto_xy(1,2);
	lcd_str("Apasati pentru a incepe:");
	
	
	lcd_goto_xy(4,4);
	lcd_str("Succes!");
	
	_delay_ms(100);
	
	while((PINC & (1 << PC1)) && (PINC & (1 << PC2)) && (PINC & (1 << PC3))){
	x++;
	_delay_ms(2000);
	}
}

/* displays a stop screen. waits until the user presses a key */
void stop_screen()
{
	char x = 0;
	lcd_clear();
	lcd_goto_xy_exact(0,0);
	lcd_goto_xy(1,1);
	lcd_chr(ARROW_LEFT);
	printf(" Sfarsit !! ");
	lcd_chr(ARROW_RIGHT);
	
	lcd_goto_xy(1,2);
	lcd_str("Apasati pentru restart.");
	
	
	lcd_goto_xy(4,4);
	
	_delay_ms(100);
	while((PINC & (1 << PC1)) && (PINC & (1 << PC2)) && (PINC & (1 << PC3))){
	x++;
	_delay_ms(2000);
	}
}

/* main -- Te fun starts here */
int main(void)
{	
	// Setup LCD
	lcd_init();
	lcd_contrast(0x40);

	initButtons();
	
	/* welcoming message */
	start_screen();
	
	
	initEnemies();
	while(1)
	{
		lcd_clear();
		
		/* check for collisions */
		collision();
		
		/* move and draw battleship*/
		move_ship();
		drawBattleship(x_ship,40);
		
		/* launch move and draw projectiles */
		launch_projectile();
		move_projectile();
		drawProjectile(x_proj, y_proj);
		
		/* draw enemies(only those who are alive) */
		drawEnemies();
		
		/* update lcd*/
		lcd_update();

		_delay_ms(300); /* wait a bit to offer it a 'natural' smooth look */
		
		move_enemies();
		
		/* check if game ended */
		if(enemies_alive == 0)
		{
			stop_screen();
			initEnemies();
		}
	}
	
	
}
